% edf2bin.m: convert an EDF file to a MATLAB-readable binary formatted file.
% As a default, data is arranged in the following column order: [lh rh lv rv],
% but other configurations (including pupil data) are possible. 
%
% Exporting data from EDF format requires the 'edf2asc' program from SR Research.
% It is freely available to registered users from their support website at:
% https://www.sr-support.com/forums/index.php
% Select the 'EyeLink Display Software' topic and from there download the version
% for your computer's operating system.  Place a copy of the 'edf2asc' executable in
% your OMtools/rd directory to enable this program (edf2bin) to perform the conversion.
%
% This program directly commands edf2asc to export the data to a '.asc' file that has
% the same name as the original .EDF file, and in the same directory.  It is done using
% the following command: edf2asc xxxx.edf -s -y -miss NaN
% Only sample data (i.e., no event information) is exported.  Dropped sample points are
% saved as 'NaN' (the IEEE representation for 'Not a Number').
%
% This program assumes that the data being imported from EDF to MATLAB was recorded 
% binocularly.  If not, you will need to know the channel order so that you can guide
% edf2bin in how to name and arrange the channels.
%

% Written by:  Jonathan Jacobs
%              August 2000 - April 2012 (last mod: 04/09/12)

% 05 Apr 12 -- edf files containing multiple trials will now be properly saved
%              in .bin format             
%              Can now deal with channels in different order than default
% 09 Apr 12 -- Now can directly read .EDF files without requiring user intervention
%              and modifications of intermediate .ASC file.
% 15 Apr 13 -- Fixed for case when EDF time index (column 1) changes from 6 digits to 7

function edf2lbin(null)

curdir = pwd;
cd(findomtools)
cd('rd')

if (exist('edf2asc','file') ~= 2)
   disp('To use this version of edf2bin, you need to have a copy of the SR Research')
   disp('executable EyeLink file "edf2asc" installed in the OMtools "rd" directory.')
   disp('If you have registered an account with SR, go to their support website at')
   disp('https://www.sr-support.com/forums/index.php and download the EyeLink Display Software')
   disp('for your platform (Mac, Mac OS X, Windows or Linux.  This will contain the needed file.')
   return
end

try
   cd(curdir)
 catch
   cd(matlabroot)
end 

[fn, pn]=uigetfile({'*.edf'}, 'Select a file to load (exten must be lower case)');
if fn == 0, disp('Aborted.'); return, end
datfile = ['''' pn fn ''''];

fname = strtok(fn,'.');

cd(findomtools)
cd('rd')
% time for the heavy lifting.
eval([ '! ./edf2asc ' datfile  ' -s -y -miss NaN' ] )
disp('EDF to ASCII conversion completed.')
disp('Importing converted data into MATLAB.  Patience is a virtue.')
raw = importdata([pn fname '.asc']);

% chop off everything after the final tab.  This will remove the non-numeric last coluumn.
disp('Data successfully loaded.  Converting to numeric values.  Tick tock, tick tock.')
for i = 1:length(raw)
   temp = raw{i};
   tabs = find(temp == 9);
   timecell{i} = raw{i}(1:tabs(1)-1);
   out{i} = raw{i}(tabs(1): tabs(end)-1);
   %out{i} = raw{i}(1: tabs(end)-1);
end
out = out';
out = cell2mat(out);

[r,c]=size(out);
for j=1:r
   out(j,:)=strrep( out(j,:), '...', 'NaN' );
end %j
out = str2num(out);
clear raw

% separate off the time column from the actual data
% have to use loop and str2double because cell2mat chokes when 6-digit time becomes
% 7-digit time during recording session.
timecell = timecell';
for i =1:length(timecell)
   t(i) =  str2double(timecell(i) );
end
%t =  out(:,1);
%in = out(:,2:end);
in = out(:,1:end);
clear out

% we need to find which channel is which
% display the number of channels, ask whether user wants to view them or
% simply enter/verify channel order.
[r,c] = size(in);
disp([ num2str(c) ' channels detected.'])
disp([ num2str(r) ' samples detected.'])

% For multiple records in a single EDF file, there will be gaps in time between each experiment.
% We will use them to separate the experiments.
tdiff=t(2:end) - t(1:end-1);
seps = find(tdiff > 100 );
disp([ num2str(length(seps)+1) ' trials detected.'])

if isempty(seps)
    % if there are no record separator lines, just
    % use the last row of the data as the end point
    disp([ 'Only 1 trial detected.'])
    seps = r;
  else    
    septrials = input('Treat as individual trials? (y/n) ','s');
    if  strcmp( lower(septrials), 'y' )
       begs = [1, (seps+1)];
       ends = [(seps-1) r];
     else
       seps = r;
    end
end

% default chan vals coming FROM edf2asc
%lh_chan=1; lv_chan=2; rh_chan=4; rv_chan=5;

% the most likely ways that the data could be read in directly from a hand-modified
% text file.  If everything went well above, the only choice that will ever be hit is 6 chan
disp(' ')

if c==6
   disp( ['Default EDF->ASC export assumption: '] )
   disp( ['   1) lh, 2) lv, 3) ?, 4) ?, 5) ?, 6) ?'])
   disp( ['   Will save in this order: [lh rh null null]'] )
   ch_err_flag=1;
   yorn = lower( input('Is this correct? (y/n) ','s') );
   if strcmp(yorn,'y')
      lh_chan=1; lv_chan=2; rh_chan=4; rv_chan=5;
      ch_err_flag = 0;
   end   
 else
   disp(['I do not know the order of the channels here.'])
   ch_err_flag = 1;
end 


% If none of the known cases exist, prompt the user for the channel names.
% I suppose it is possible that we might need this if the data were taken in
% an unusual form, perhaps monocularly.
strarray = [{'lh'},{'rh'},{'lv'},{'rv'},{'lp'},{'rp'}];
%% name the channels
if ch_err_flag
   sampfreq = input('Enter the sampling frequency: ');
   for i=1:c
      chtemp = in(:,i); t=maket(chtemp,sampfreq);
      figure; plot(t,chtemp)
      commandwindow
     chname{i} = input(['Enter a name for channel ' num2str(i) '.  Enter "-" to ignore it: '],'s');
     chpos = strmatch(chname{i}, strarray);
      if chpos
        eval( [strarray{chpos} '_chan = i;'])
        %disp(['   Assigning channel ' str2num(i) ' as ' chname{i} '.'] )
      end   
   end
end


% Arange the data so that it can be written to a binary file.
fname=lower(fname);
cd(pn)

% two cases:
% 1) single record/ EDF file.  save it as xxx_1.bin
clear temp
if length(seps)==1
   dat = [];
   if exist('lh_chan'), dat = [in(:,lh_chan)]; end
   if exist('rh_chan'), dat = [dat, in(:,rh_chan)]; end
   if exist('lv_chan'), dat = [dat, -in(:,lv_chan)]; end
   if exist('rv_chan'), dat = [dat, -in(:,rv_chan)]; end
   
   temp{1} = [fname '.bin'];
   disp(['saving ' '''' pn temp{1} ''''])
   fid = fopen(temp{1}, 'w', 'l');
   fwrite(fid, dat, 'float');
   fclose(fid);
   cd(curdir)
 else
   % 2) multiple recordings in an EDF file
   % extract them all and save as xxx_1, xxx_2, etc...
   dat = [];
   for x = 1:length(seps)+1
     if exist('lh_chan'), dat = [in(begs(x):ends(x),lh_chan)]; end
     if exist('rh_chan'), dat = [dat,  in(begs(x):ends(x),rh_chan)]; end
     if exist('lv_chan'), dat = [dat, -in(begs(x):ends(x),lv_chan)]; end
     if exist('rv_chan'), dat = [dat, -in(begs(x):ends(x),rv_chan)]; end
     
      temp{x} = [fname '_' num2str(x) '.bin'];       
      disp(['saving ' '''' pn temp{x} ''''])
      fid = fopen(temp{x}, 'w', 'l');
      fwrite(fid, dat, 'float');
      fclose(fid);
   end
end

try
   cd(curdir)
 catch
   cd(matlabroot)
end  

disp(' ')
disp('Remember that you will need an adjustbias file to read the data using ''rd''.')
disp('Type ''biasgen'' to create the adjustbias file.   You will need to know the')
disp('sampling frequency used to take the data, as well as which channels were recorded.')
disp('When ''biasgen'' asks, the Method is (V)ideo and the Format is (B)inary,')
disp('the Calibration is (N)ormal and the Channel Structure is (C)ontiguous.')